Index: channels/chan_zap.c
===================================================================
--- channels/chan_zap.c	(revision 86182)
+++ channels/chan_zap.c	(working copy)
@@ -7163,8 +7163,10 @@
 		/* Lock the interface list */
 		ast_mutex_lock(&iflock);
 		if (!pfds || (lastalloc != ifcount)) {
-			if (pfds)
+			if (pfds) {
 				ast_free(pfds);
+				pfds = NULL;
+			}
 			if (ifcount) {
 				if (!(pfds = ast_calloc(1, ifcount * sizeof(*pfds)))) {
 					ast_mutex_unlock(&iflock);
@@ -10863,16 +10865,17 @@
 }
 
 static int setup_zap(int reload);
+static void destroy_all_channels(void);
+
 static int zap_restart(void)
 {
 	ast_verb(1, "Destroying channels and reloading zaptel configuration.\n");
-	while (iflist) {
-		ast_debug(1, "Destroying zaptel channel no. %d\n", iflist->channel);
-		/* Also updates iflist: */
-		destroy_channel(NULL, iflist, 1);
-	}
+	destroy_all_channels();
+
+	/* FIXME: We need to wait "for a while" after destroying channels. */
+	sleep(1);
 	ast_debug(1, "Channels destroyed. Now re-reading config.\n");
-	if (setup_zap(2) != 0) {
+	if (setup_zap(0) != 0) {
 		ast_log(LOG_WARNING, "Reload channels from zap config failed!\n");
 		return 1;
 	}
@@ -11541,11 +11544,38 @@
 	return 0;
 }
 
-static int __unload_module(void)
+static void destroy_all_channels(void)
 {
 	int x;
 	struct zt_pvt *p, *pl;
 
+	ast_mutex_lock(&iflock);
+	/* Destroy all the interfaces and free their memory */
+	p = iflist;
+	while (p) {
+		/* Free any callerid */
+		if (p->cidspill)
+			ast_free(p->cidspill);
+		/* Close the zapata thingy */
+		if (p->subs[SUB_REAL].zfd > -1)
+			zt_close(p->subs[SUB_REAL].zfd);
+		pl = p;
+		p = p->next;
+		x = pl->channel;
+		/* Free associated memory */
+		if (pl)
+			destroy_zt_pvt(&pl);
+		ast_verb(3, "Unregistered channel %d\n", x);
+	}
+	iflist = NULL;
+	ifcount = 0;
+	ast_mutex_unlock(&iflock);
+}
+
+static int __unload_module(void)
+{
+	struct zt_pvt *p;
+
 #ifdef HAVE_PRI
 	int i;
 	for (i = 0; i < NUM_SPANS; i++) {
@@ -11582,27 +11612,7 @@
 	monitor_thread = AST_PTHREADT_STOP;
 	ast_mutex_unlock(&monlock);
 
-	ast_mutex_lock(&iflock);
-	/* Destroy all the interfaces and free their memory */
-	p = iflist;
-	while (p) {
-		/* Free any callerid */
-		if (p->cidspill)
-			ast_free(p->cidspill);
-		/* Close the zapata thingy */
-		if (p->subs[SUB_REAL].zfd > -1)
-			zt_close(p->subs[SUB_REAL].zfd);
-		pl = p;
-		p = p->next;
-		x = pl->channel;
-		/* Free associated memory */
-		if (pl)
-			destroy_zt_pvt(&pl);
-		ast_verb(3, "Unregistered channel %d\n", x);
-	}
-	iflist = NULL;
-	ifcount = 0;
-	ast_mutex_unlock(&iflock);
+	destroy_all_channels();
 #ifdef HAVE_PRI		
 	for (i = 0; i < NUM_SPANS; i++) {
 		if (pris[i].master && (pris[i].master != AST_PTHREADT_NULL))
